@@ -1,10 +1,10 @@
-# $Id: HTML.pm,v 1.64.2.1 2003/03/18 21:55:25 felicity Exp $
+# $Id: HTML.pm,v 1.96 2003/09/12 00:04:27 quinlan Exp $
 
 package Mail::SpamAssassin::HTML;
 1;
 
 package Mail::SpamAssassin::PerMsgStatus;
-use HTML::Parser 3.00 ();
+use HTML::Parser 3.24 ();
 
 use strict;
 use bytes;
@@ -23,6 +23,18 @@ $re_strict = 'a|abbr|acronym|address|area|b|base|bdo|big|blockquote|body|br|butt
 # loose list of HTML events
 $events = 'on(?:activate|afterupdate|beforeactivate|beforecopy|beforecut|beforedeactivate|beforeeditfocus|beforepaste|beforeupdate|blur|change|click|contextmenu|controlselect|copy|cut|dblclick|deactivate|errorupdate|focus|focusin|focusout|help|keydown|keypress|keyup|load|losecapture|mousedown|mouseenter|mouseleave|mousemove|mouseout|mouseover|mouseup|mousewheel|move|moveend|movestart|paste|propertychange|readystatechange|reset|resize|resizeend|resizestart|select|submit|timeerror|unload)';
 
+my %tested_colors;
+
+sub html_init {
+  my ($self) = @_;
+
+  push @{ $self->{bgcolor_color} }, "#ffffff";
+  push @{ $self->{bgcolor_tag} }, "default";
+  push @{ $self->{fgcolor_color} }, "#000000";
+  push @{ $self->{fgcolor_tag} }, "default";
+  undef %tested_colors;
+}
+
 sub html_tag {
   my ($self, $tag, $attr, $num) = @_;
 
@@ -31,6 +43,13 @@ sub html_tag {
   $self->{html}{elements}++ if $tag =~ /^(?:$re_strict|$re_loose)$/io;
   $self->{html}{tags}++;
 
+  if ($tag =~ /^(?:body|table|tr|th|td)$/) {
+    $self->html_bgcolor($tag, $attr, $num);
+  }
+  if ($tag =~ /^(?:body|font)$/) {
+    $self->html_fgcolor($tag, $attr, $num);
+  }
+
   if ($num == 1) {
     $self->html_format($tag, $attr, $num);
     $self->html_uri($tag, $attr, $num);
@@ -141,28 +160,198 @@ sub rgb_to_hsv {
   return ($h, $s, $v);
 }
 
-# the most common HTML colors
-my %name_to_rgb = (
-  red           => '#ff0000',
-  black         => '#000000',
-  blue          => '#0000ff',
-  white         => '#ffffff',
-  navy          => '#000080',
-  green         => '#008000',
-  orange        => '#ffa500',
-  yellow        => '#ffff00',
-  fuchsia       => '#ff00ff',
-  lime          => '#00ff00',
-  maroon        => '#800000',
-  darkblue      => '#00008b',
-  gray          => '#808080',
-  purple        => '#800080',
-  magenta       => '#ff00ff',
-  pink          => '#ffc0cb',
+# HTML 4 defined 16 colors
+my %html_color = (
+  aqua		=> '#00ffff',
+  black		=> '#000000',
+  blue		=> '#0000ff',
+  fuchsia	=> '#ff00ff',
+  gray		=> '#808080',
+  green		=> '#008000',
+  lime		=> '#00ff00',
+  maroon	=> '#800000',
+  navy		=> '#000080',
+  olive		=> '#808000',
+  purple	=> '#800080',
+  red		=> '#ff0000',
+  silver	=> '#c0c0c0',
+  teal		=> '#008080',
+  white		=> '#ffffff',
+  yellow	=> '#ffff00',
+);
+
+# popular X11 colors specified in CSS3 color module
+my %name_color = (
+  aliceblue	=> '#f0f8ff',
+  cyan		=> '#00ffff',
+  darkblue	=> '#00008b',
+  darkcyan	=> '#008b8b',
+  darkgray	=> '#a9a9a9',
+  darkgreen	=> '#006400',
+  darkred	=> '#8b0000',
+  firebrick	=> '#b22222',
+  gold		=> '#ffd700',
+  lightslategray=> '#778899',
+  magenta	=> '#ff00ff',
+  orange	=> '#ffa500',
+  pink		=> '#ffc0cb',
+  whitesmoke	=> '#f5f5f5',
 );
 
 sub name_to_rgb {
-  return $name_to_rgb{$_[0]} || $_[0];
+  return $html_color{$_[0]} || $name_color{$_[0]} || $_[0];
+}
+
+sub pop_bgcolor {
+  my ($self) = @_;
+
+  pop @{ $self->{bgcolor_color} };
+  pop @{ $self->{bgcolor_tag} };
+}
+
+sub html_bgcolor {
+  my ($self, $tag, $attr, $num) = @_;
+
+  if ($num == 1) {
+    # close elements with optional end tags
+    if ($tag eq "body") {
+      # compromise between HTML browsers generally only using first
+      # body and some messages including multiple HTML attachments:
+      # pop everything except first body color
+      while ($self->{bgcolor_tag}[-1] !~ /^(?:default|body)$/) {
+	$self->pop_bgcolor();
+      }
+    }
+    if ($tag eq "tr") {
+      while ($self->{bgcolor_tag}[-1] =~ /^t[hd]$/) {
+	$self->pop_bgcolor();
+      }
+      $self->pop_bgcolor() if $self->{bgcolor_tag}[-1] eq "tr";
+    }
+    elsif ($tag =~ /^t[hd]$/) {
+      $self->pop_bgcolor() if $self->{bgcolor_tag}[-1] =~ /^t[hd]$/;
+    }
+    # figure out new bgcolor
+    my $bgcolor;
+    if (exists $attr->{bgcolor}) {
+      $bgcolor = name_to_rgb(lc($attr->{bgcolor}));
+    }
+    else {
+      $bgcolor = $self->{bgcolor_color}[-1];
+    }
+    # tests
+    if ($tag eq "body" && $bgcolor !~ /^\#?ffffff$/) {
+      $self->{html}{bgcolor_nonwhite} = 1;
+    }
+    # push new bgcolor
+    push @{ $self->{bgcolor_color} }, $bgcolor;
+    push @{ $self->{bgcolor_tag} }, $tag;
+  }
+  else {
+    # close elements
+    if ($tag eq "body") {
+      $self->pop_bgcolor() if $self->{bgcolor_tag}[-1] eq "body";
+    }
+    elsif ($tag eq "table") {
+      while ($self->{bgcolor_tag}[-1] =~ /^t[rhd]$/) {
+	$self->pop_bgcolor();
+      }
+      $self->pop_bgcolor() if $self->{bgcolor_tag}[-1] eq "table";
+    }
+    elsif ($tag eq "tr") {
+      while ($self->{bgcolor_tag}[-1] =~ /^t[hd]$/) {
+	$self->pop_bgcolor();
+      }
+      $self->pop_bgcolor() if $self->{bgcolor_tag}[-1] eq "tr";
+    }
+    elsif ($tag =~ /^t[hd]$/) {
+      $self->pop_bgcolor() if $self->{bgcolor_tag}[-1] =~ /^t[hd]$/;
+    }
+  }
+}
+  
+sub pop_fgcolor {
+  my ($self) = @_;
+
+  pop @{ $self->{fgcolor_color} };
+  pop @{ $self->{fgcolor_tag} };
+}
+
+sub html_fgcolor {
+  my ($self, $tag, $attr, $num) = @_;
+
+  if ($num == 1) {
+    if ($tag eq "body") {
+      # compromise between HTML browsers generally only using first
+      # body and some messages including multiple HTML attachments:
+      # pop everything except first body color
+      while ($self->{fgcolor_tag}[-1] !~ /^(?:default|body)$/) {
+	$self->pop_fgcolor();
+      }
+    }
+    # figure out new fgcolor
+    my $fgcolor;
+    if ($tag eq "font" && exists $attr->{color}) {
+      $fgcolor = name_to_rgb(lc($attr->{color}));
+    }
+    elsif ($tag eq "body" && exists $attr->{text}) {
+      $fgcolor = name_to_rgb(lc($attr->{text}));
+    }
+    else {
+      $fgcolor = $self->{fgcolor_color}[-1];
+    }
+    # push new fgcolor
+    push @{ $self->{fgcolor_color} }, $fgcolor;
+    push @{ $self->{fgcolor_tag} }, $tag;
+  }
+  else {
+    # close elements
+    if ($tag eq "body") {
+      $self->pop_fgcolor() if $self->{fgcolor_tag}[-1] eq "body";
+    }
+    if ($tag eq "font") {
+      $self->pop_fgcolor() if $self->{fgcolor_tag}[-1] eq "font";
+    }
+  }
+}
+
+sub html_font_invisible {
+  my ($self, $text) = @_;
+
+  my $fg = $self->{fgcolor_color}[-1];
+  my $bg = $self->{bgcolor_color}[-1];
+
+  return if exists $tested_colors{"$fg\000$bg"};
+  $tested_colors{"$fg\000$bg"}++;
+
+  # invisibility
+  if (substr($fg,-6) eq substr($bg,-6)) {
+    $self->{html}{font_invisible} = 1;
+  }
+  # near-invisibility
+  elsif ($fg =~ /^\#?([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})$/) {
+    my ($r1, $g1, $b1) = (hex($1), hex($2), hex($3));
+
+    if ($bg =~ /^\#?([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})$/) {
+      my ($r2, $g2, $b2) = (hex($1), hex($2), hex($3));
+
+      my $r = ($r1 - $r2);
+      my $g = ($g1 - $g2);
+      my $b = ($b1 - $b2);
+
+      # geometric distance weighted by brightness
+      # maximum distance is 191.151823601032
+      my $distance = ((0.2126*$r)**2 + (0.7152*$g)**2 + (0.0722*$b)**2)**0.5;
+
+      # the text is very difficult to read if the distance is under 12,
+      # a limit of 14 to 16 might be okay if the usage significantly
+      # increases (near-invisible text is at about 0.95% of spam and
+      # 1.25% of HTML spam right now), but please test any changes first
+      if ($distance < 12) {
+	$self->{html}{"font_near_invisible"} = 1;
+      }
+    }
+  }
 }
 
 sub html_tests {
@@ -181,7 +370,8 @@ sub html_tests {
       {
 	$self->{html}{html_event} = 1;
       }
-      if (/\bon(?:blur|contextmenu|focus|load|resize|submit|unload)\b/i)
+      if (/\bon(?:blur|contextmenu|focus|load|resize|submit|unload)\b/i &&
+	  $attr->{$_})
       {
 	$self->{html}{html_event_unsafe} = 1;
         if ($attr->{$_} =~ /\.open\s*\(/) { $self->{html}{window_open} = 1; }
@@ -190,26 +380,22 @@ sub html_tests {
       }
     }
   }
-  if ($tag eq "body" && exists $attr->{bgcolor}) {
-    $self->{html}{bgcolor} = lc($attr->{bgcolor});
-    $self->{html}{bgcolor} = name_to_rgb($self->{html}{bgcolor});
-    $self->{html}{bgcolor_nonwhite} = 1 if $self->{html}{bgcolor} !~ /^\#?ffffff$/;
-  }
   if ($tag eq "font" && exists $attr->{size}) {
     $self->{html}{big_font} = 1 if (($attr->{size} =~ /^\s*(\d+)/ && $1 > 3) ||
 			    ($attr->{size} =~ /\+(\d+)/ && $1 >= 1));
   }
   if ($tag eq "font" && exists $attr->{color}) {
-    my $c = lc($attr->{color});
-    $self->{html}{font_color_nohash} = 1 if $c =~ /^[0-9a-f]{6}$/;
-    $self->{html}{font_color_unsafe} = 1 if ($c =~ /^\#?[0-9a-f]{6}$/ &&
-				     $c !~ /^\#?(?:00|33|66|80|99|cc|ff){3}$/);
-    $self->{html}{font_color_name} = 1 if ($c !~ /^\#?[0-9a-f]{6}$/ &&
-				   $c !~ /^(?:navy|gray|red|white)$/);
-    $c = name_to_rgb($c);
-    $self->{html}{font_invisible} = 1 if (exists $self->{html}{bgcolor} &&
-                                substr($c,-6) eq substr($self->{html}{bgcolor},-6));
-    if ($c =~ /^\#?([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})$/) {
+    my $bg = $self->{bgcolor_color}[-1];
+    my $fg = lc($attr->{color});
+    if ($fg =~ /^\#?[0-9a-f]{6}$/ && $fg !~ /^\#?(?:00|33|66|80|99|cc|ff){3}$/)
+    {
+      $self->{html}{font_color_unsafe} = 1;
+    }
+    if ($fg !~ /^\#?[0-9a-f]{6}$/ && !exists $html_color{$fg})
+    {
+      $self->{html}{font_color_name} = 1;
+    }
+    if ($fg =~ /^\#?([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})$/) {
       my ($h, $s, $v) = rgb_to_hsv(hex($1), hex($2), hex($3));
       if (!defined($h)) {
 	$self->{html}{font_gray} = 1 unless ($v == 0 || $v == 255);
@@ -251,25 +437,16 @@ sub html_tests {
   }
   if (exists($attr->{style})) {
     if ($attr->{style} =~ /font(?:-size)?:\s*(\d+(?:\.\d*)?|\.\d+)(p[tx])/i) {
-      my $size = $1+0;
+      my $size = $1;
       my $type = $2;
 
-      $self->{html}{big_font_B} = 1 if (lc($type) eq "pt" && $size > 12);
-    }
-  }
-  if (($tag eq "img" &&
-       exists $attr->{src} && ($_ = $attr->{src})) ||
-      ($tag =~ /^(?:body|table|tr|td|th)$/ && 
-       exists $attr->{background} && ($_ = $attr->{background})))
-  {
-    if (/\?/ || (/[a-f\d]{12,}/i && ! /\.(?:jpe?g|gif|png)$/i && !/^cid:/))
-    {
-      $self->{html}{web_bugs} = 1;
+      $self->{html}{big_font} = 1 if (lc($type) eq "pt" && $size > 12);
     }
   }
   if ($tag eq "img" && exists $attr->{width} && exists $attr->{height}) {
     my $width = 0;
     my $height = 0;
+    my $area = 0;
 
     # assume 800x600 screen for percentage values
     if ($attr->{width} =~ /^(\d+)(\%)?$/) {
@@ -281,9 +458,15 @@ sub html_tests {
       $height *= 6 if (defined $2 && $2 eq "%");
     }
     if ($width > 0 && $height > 0) {
-      my $area = $width * $height;
+      $area = $width * $height;
       $self->{html}{image_area} += $area;
     }
+    # this is intended to match any width and height if they're specified
+    if (exists $attr->{src} &&
+	$attr->{src} =~ /\.(?:pl|cgi|php|asp|jsp|cfm)\b/i)
+    {
+      $self->{html}{web_bugs} = 1;
+    }
   }
   if ($tag eq "form" && exists $attr->{action}) {
     $self->{html}{form_action_mailto} = 1 if $attr->{action} =~ /mailto:/i
@@ -339,7 +522,7 @@ sub html_text {
       my $size = $1;
       my $type = $2;
 
-      $self->{html}{big_font_B} = 1 if (lc($type) eq "pt" && $size > 12);
+      $self->{html}{big_font} = 1 if (lc($type) eq "pt" && $size > 12);
     }
     return;
   }
@@ -350,6 +533,8 @@ sub html_text {
     $self->{html}{title_text} .= $text;
   }
 
+  $self->html_font_invisible($text) if $text =~ /[^ \t\n\r\f\x0b\xa0]/;
+
   $text =~ s/^\n//s if $self->{html_last_tag} eq "br";
   push @{$self->{html_text}}, $text;
 }
@@ -362,6 +547,7 @@ sub html_comment {
   $self->{html}{comment_egp} = 1 if $text =~ /\S+begin egp html banner\S+/;
   $self->{html}{comment_saved_url} = 1 if $text =~ /<!-- saved from url=\(\d{4}\)/;
   $self->{html}{comment_sky} = 1 if $text =~ /SKY-(?:Email-Address|Database|Mailing|List)/;
+  $self->{html}{total_comment_length} += length($text) + 7; # "<!--" + "-->"
 
   if (exists $self->{html_inside}{script} && $self->{html_inside}{script} > 0)
   {
@@ -384,9 +570,25 @@ sub html_comment {
       my $size = $1;
       my $type = $2;
 
-      $self->{html}{big_font_B} = 1 if (lc($type) eq "pt" && $size > 12);
+      $self->{html}{big_font} = 1 if (lc($type) eq "pt" && $size > 12);
     }
   }
+
+  if (exists $self->{html}{shouting} && $self->{html}{shouting} > 1) {
+    $self->{html}{comment_shouting} = 1;
+  }
+}
+
+sub html_declaration {
+  my ($self, $text) = @_;
+
+  if ($text =~ /^<!doctype/i) {
+    my $tag = "!doctype";
+
+    $self->{html}{elements}++;
+    $self->{html}{tags}++;
+    $self->{html_inside}{$tag} = 0;
+  }
 }
 
 ###########################################################################
