@@ -18,21 +18,32 @@ use vars qw{
 # A good listing is in /usr/share/config/charsets from KDE 2.2.1
 #
 %charsets_for_locale = (
-  # Japanese
-  'ja' => 'EUCJP JISX020119760 JISX020819830 JISX020819900 JISX020819970 JISX021219900 JISX021320001 JISX021320002 SHIFT_JIS SHIFTJIS ISO2022JP SJIS JIS7',
+
+  # Japanese: Peter Evans writes: iso-2022-jp = rfc approved, rfc 1468, created
+  # by Jun Murai in 1993 back when he didnt have white hair!  rfc approved.
+  # (rfc 2237) <-- by M$. 
+  'ja' => 'EUCJP JISX020119760 JISX020819830 JISX020819900 JISX020819970 '.
+	'JISX021219900 JISX021320001 JISX021320002 SHIFT_JIS SHIFTJIS '.
+	'ISO2022JP SJIS JIS7 JISX0201 JISX0208 JISX0212',
+
   # Korea
-  'ko' => 'EUCKR EUCKR',
-  # Cyrillic
-  'ru' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154',
-  'ka' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154',
-  'tg' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154',
-  'be' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154',
-  'uk' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154',
-  'bg' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154',
+  'ko' => 'EUCKR KSC56011987',
+
+  # Cyrillic: Andrew Vasilyev notes CP866 is common (bug 2278)
+  'ru' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154 CP866',
+  'ka' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154 CP866',
+  'tg' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154 CP866',
+  'be' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154 CP866',
+  'uk' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154 CP866',
+  'bg' => 'KOI8R KOI8U KOI8T ISOIR111 CP1251 GEORGIANPS CP1251 PT154 CP866',
+
   # Thai
   'th' => 'TIS620',
-  # Chinese (simplified and traditional)
-  'zh' => 'GB2312 GB231219800 GB18030 GBK BIG5HKSCS BIG5 EUCTW',
+
+  # Chinese (simplified and traditional).   Peter Evans writes: new government
+  # mandated chinese encoding = gb18030, chinese mail is supposed to be
+  # iso-2022-cn (rfc 1922?)
+  'zh' => 'GB1988 GB2312 GB231219800 GB18030 GBK BIG5HKSCS BIG5 EUCTW ISO2022CN',
 );
 
 ###########################################################################
@@ -74,3 +85,5 @@ sub is_charset_ok_for_locales {
 
   return 0;
 }
+
+1;
