@@ -109,7 +109,7 @@ sub parse_headers {
       $self->{from_line} = $_;
       next;
 
-    } elsif (/^([^\x00-\x20\x7f-\xff:]+):\s*(.*)$/) {
+    } elsif (/^([^\x00-\x20\x7f-\xff:]+):\s*(.*)$/s) {
       $hdr = $1; $val = $2;
       $val =~ s/\r+//gs;          # trim CRs, we don't want them
       $entry = $self->_get_or_create_header_object ($hdr);
@@ -128,13 +128,21 @@ sub parse_headers {
 }
 
 sub _add_header_to_entry {
-  my ($self, $entry, $hdr, $line) = @_;
+  my ($self, $entry, $hdr, $line, $order) = @_;
+
+  # Do a normal push if no specific order # is set.
+  $order ||= @{$self->{header_order}};
 
   # ensure we have line endings
   $line .= "\n" unless $line =~ /\n$/;
 
+  # Store this header
   $entry->{$entry->{count}} = $line;
-  push (@{$self->{header_order}}, $hdr.":".$entry->{count});
+
+  # Push the header and which count it is in header_order
+  splice @{$self->{header_order}}, $order, 0, $hdr.":".$entry->{count};
+
+  # Increase the count of this header type
   $entry->{count}++;
 }
 
@@ -173,8 +181,13 @@ sub _get_header_list {
 
 sub get_pristine_header {
   my ($self, $hdr) = @_;
-  my($ret) = $self->{headers_pristine} =~ /^(?:$hdr:[ ]+(.*\n(?:\s+\S.*\n)*))/mi;
-  return ( $ret || $self->get_header($hdr) );
+  my(@ret) = $self->{headers_pristine} =~ /^(?:$hdr:[ ]+(.*\n(?:\s+\S.*\n)*))/mig;
+  if (@ret) {
+    return wantarray ? @ret : $ret[0];
+  }
+  else {
+    return $self->get_header($hdr);
+  }
 }
 
 sub get_header {
@@ -217,10 +230,10 @@ sub get_header {
 }
 
 sub put_header {
-  my ($self, $hdr, $text) = @_;
+  my ($self, $hdr, $text, $order) = @_;
 
   my $entry = $self->_get_or_create_header_object ($hdr);
-  $self->_add_header_to_entry ($entry, $hdr, $text);
+  $self->_add_header_to_entry ($entry, $hdr, $text, $order);
   if (!$entry->{original}) { $entry->{added} = 1; }
 }
 
@@ -254,12 +267,27 @@ sub get_all_headers {
 sub replace_header {
   my ($self, $hdr, $text) = @_;
 
-  # perhaps we should check $self->{header_order} and put the new version
-  # where the old one was?  This shouldn't be required anywhere though.
-  # tvd, 2003.02.23
+  # Figure out where the first case insensitive header of this name is stored.
+  # We'll use this to add the new header with the same case and in the order.
+  my($casehdr,$order) = ($hdr,undef);
+  my $lchdr = lc "$hdr:0"; # just lc it once
 
+  # Now find the header
+  for ( my $count = 0; $count <= @{$self->{header_order}}; $count++ ) {
+    next unless (lc $self->{header_order}->[$count] eq $lchdr);
+
+    # Remember where in the order the header is, and the case of said header.
+    $order = $count;
+    ($casehdr = $self->{header_order}->[$count]) =~ s/:\d+$//;
+
+    last;
+  }
+
+  # Remove all instances of this header
   $self->delete_header ($hdr);
-  return $self->put_header($hdr, $text);
+
+  # Add the new header with correctly cased header and in the right place
+  return $self->put_header($casehdr, $text, $order);
 }
 
 sub delete_header {
@@ -267,7 +295,7 @@ sub delete_header {
 
   # Delete all versions of the header, case insensitively
   foreach my $dhdr ( $self->_get_header_list($hdr,1) ) {
-    @{$self->{header_order}} = grep(rindex($_,"$dhdr:",0) != 0, @{$self->{header_order}});
+    @{$self->{header_order}} = grep( rindex($_,"$dhdr:",0) != 0, @{$self->{header_order}} );
     delete $self->{headers}->{$dhdr};
   }
 }
@@ -290,6 +318,11 @@ sub get_pristine {
   return join ('', $self->{headers_pristine}, @{ $self->{textarray} });
 }
 
+sub get_pristine_body {
+  my ($self) = @_;
+  return join ('', @{ $self->{textarray} });
+}
+
 sub as_string {
   my ($self) = @_;
   return join ('', $self->get_all_headers(), "\n",
