@@ -652,14 +652,16 @@ sub parse_received_line {
     }
 
     # Received: from ironport.com (10.1.1.5) by a50.ironport.com with ESMTP; 01 Apr 2003 12:00:51 -0800
+    # Received: from dyn-81-166-39-132.ppp.tiscali.fr (81.166.39.132) by cpmail.dk.tiscali.com (6.7.018)
     # note: must be before 'Content Technologies SMTPRS' rule, cf. bug 2787
-    if (/^from (\S+) \((${IP_ADDRESS})\) by (\S+) with /) {
+    if (/^from (\S+) \((${IP_ADDRESS})\) by (\S+) /) {
       $helo = $1; $ip = $2; $by = $3; goto enough;
     }
 
     # Received: from scv3.apple.com (scv3.apple.com) by mailgate2.apple.com (Content Technologies SMTPRS 4.2.1) with ESMTP id <T61095998e1118164e13f8@mailgate2.apple.com>; Mon, 17 Mar 2003 17:04:54 -0800
     if (/^from (\S+) \((\S+)\) by (\S+) \(/) {
-      $helo = $1; $rdns = $2; $by = $3; goto enough;
+      return;		# useless without the $ip anyway!
+      #$helo = $1; $rdns = $2; $by = $3; goto enough;
     }
 
     # Received: from 01al10015010057.ad.bls.com ([90.152.5.141] [90.152.5.141])
@@ -667,14 +669,14 @@ sub parse_received_line {
     if (/^from (\S+) \(\[(\S+)\] \[(\S+)\]\) by (\S+) with /) {
       # not sure what $3 is ;)
       $helo = $1; $ip = $2; $by = $4;
-	goto enough;
+      goto enough;
     }
 
     # Received: from 206.47.0.153 by dm3cn8.bell.ca with ESMTP (Tumbleweed MMS
     # SMTP Relay (MMS v5.0)); Mon, 24 Mar 2003 19:49:48 -0500
     if (/^from (${IP_ADDRESS}) by (\S+) with /) {
       $ip = $1; $by = $2;
-	goto enough;
+      goto enough;
     }
 
     # Received: from pobox.com (h005018086b3b.ne.client2.attbi.com[66.31.45.164])
@@ -812,30 +814,35 @@ sub parse_received_line {
   if (/^helo=(\S+)[^-A-Za-z0-9\.]/) { $helo = $1; }
   if (/\[(${IP_ADDRESS})\]/) { $ip = $1; }
   if (/ by (\S+)[^-A-Za-z0-9\.]/) { $by = $1; }
-  if (defined $ip && defined $by) { goto enough; }
+  if ($ip && $by) { goto enough; }
 
   # ------------------------------------------------------------------------
   # OK, if we still haven't figured out at least the basics (IP and by), or
   # returned due to it being a known-crap format, let's warn so the user can
   # file a bug report or something.
 
-  if (!defined $ip || !defined $by) {
-    dbg ("received-header: unknown format: $_");
-    # and skip the line entirely!  We can't parse it...
-    return;
-  }
+  dbg ("received-header: unknown format: $_");
+  # and skip the line entirely!  We can't parse it...
+  return;
 
   # ------------------------------------------------------------------------
   # OK, line parsed (at least partially); now deal with the contents
 
 enough:
 
+  # flag handovers we couldn't get an IP address from at all
+  if ($ip eq '') {
+    dbg ("received-header: could not parse IP address from: $_");
+  }
+
   $ip = Mail::SpamAssassin::Util::extract_ipv4_addr_from_string ($ip);
-  if (!defined $ip) {
+  if (!$ip) {
+    dbg ("received-header: could not parse IPv4 address, assuming IPv6");
     return;	# ignore IPv6 handovers
   }
 
   if ($ip eq '127.0.0.1') {
+    dbg ("received-header: ignoring localhost handover");
     return;	# ignore localhost handovers
   }
 
